<?php

/**
 * Maintenance script to import one or more images from the local file system into
 * the wiki without using the web-based interface.
 *
 * "Smart import" additions:
 * - aim: preserve the essential metadata (user, description) when importing medias from an existing wiki
 * - process:
 *      - interface with the source wiki, don't use bare files only (see --source_wiki_url)
 *      - fetch metadata from source wiki for each file to import
 *      - commit the fetched metadata to the destination wiki while submitting (through wiki's API)
 *
 * @file
 * @ingroup Maintenance
 * @author Rob Church <robchur@gmail.com>
 * @author Mij <mij@bitchx.it>
 */


$optionsWithArgs = array( 'extensions', 'comment', 'comment-file', 'comment-ext', 'user', 'license', 'source_wiki_url' );
require_once( 'commandLine.inc' );
require_once( 'myimportImages.inc.php' );
$added = $skipped = $overwritten = 0;

echo( "Import Images\n\n" );

# Need a path
if( count( $args ) > 0 ) {

	$dir = $args[0];

	# Prepare the list of allowed extensions
	global $wgFileExtensions;
	$extensions = isset( $options['extensions'] )
		? explode( ',', strtolower( $options['extensions'] ) )
		: $wgFileExtensions;

	# Search the path provided for candidates for import
	$files = findFiles( $dir, $extensions );

	# Initialise the user for this operation
	$user = isset( $options['user'] )
		? User::newFromName( $options['user'] )
		: User::newFromName( 'Maintenance script' );
	if( !$user instanceof User )
		$user = User::newFromName( 'Maintenance script' );
	$wgUser = $user;

	# Get the upload comment
	$comment = 'Importing image file';

	if ( isset( $options['comment-file'] ) ) {
		$comment =  file_get_contents( $options['comment-file'] );
		if ( $comment === false || $comment === NULL ) {
			die( "failed to read comment file: {$options['comment-file']}\n" );
		}
	}
	else if ( isset( $options['comment'] ) ) {
		$comment =  $options['comment'];
	}

	$commentExt = isset( $options['comment-ext'] ) ? $options['comment-ext'] : false;

	# Get the license specifier
	$license = isset( $options['license'] ) ? $options['license'] : '';

	# Batch "upload" operation
	if( ( $count = count( $files ) ) > 0 ) {
	
		foreach( $files as $file ) {
			$base = wfBaseName( $file );
	
			# Validate a title
			$title = Title::makeTitleSafe( NS_FILE, $base );
			if( !is_object( $title ) ) {
				echo( "{$base} could not be imported; a valid title cannot be produced\n" );
				continue;
			}
	
			# Check existence
			$image = wfLocalFile( $title );
			if( $image->exists() ) {
				if( isset( $options['overwrite'] ) ) {
					echo( "{$base} exists, overwriting..." );
					$svar = 'overwritten';
				} else {
					echo( "{$base} exists, skipping\n" );
					$skipped++;
					continue;
				}
			} else {
				echo( "Importing {$base}..." );
				$svar = 'added';
			}

            if (isset( $options['source_wiki_url'])) {
                /* find comment text directly from source wiki, through MW's API */
                $real_comment = getFileCommentFromSourceWiki($options['source_wiki_url'], $base);
                if ($real_comment === false)
                    $commentText = $comment;
                else
                    $commentText = $real_comment;

                /* find user directly from source wiki, through MW's API */
                $real_user = getFileUserFromSourceWiki($options['source_wiki_url'], $base);
                if ($real_user === false) {
                    $wgUser = $user;
                } else {
                    $wgUser = User::newFromName($real_user);
                    if ($wgUser === false) {
                        # user does not exist in target wiki
                        echo ("failed: user '$real_user' does not exist in target wiki.");
                        continue;
                    }
                }
            } else {
                # Find comment text
                $commentText = false;

                if ( $commentExt ) {
                    $f = findAuxFile( $file, $commentExt );
                    if ( !$f ) {
                        echo( " No comment file with extension {$commentExt} found for {$file}, using default comment. " );
                    } else {
                        $commentText = file_get_contents( $f );
                        if ( !$f ) {
                            echo( " Failed to load comment file {$f}, using default comment. " );
                        }
                    }
                }

                if ( !$commentText ) {
                    $commentText = $comment;
                }
            }


			# Import the file	
			if ( isset( $options['dry'] ) ) {
				echo( " publishing {$file} by '$real_user', comment '$commentText'... " );
			} else {
				$archive = $image->publish( $file );
				if( WikiError::isError( $archive ) || !$archive->isGood() ) {
					echo( "failed.\n" );
					continue;
				}
			}

			$$svar++;
			if ( isset( $options['dry'] ) ) {
				echo( "done.\n" );
			} else if ( $image->recordUpload( $archive->value, $commentText, $license ) ) {
				# We're done!
				echo( "done.\n" );
			} else {
				echo( "failed.\n" );
			}
			
		}
		
		# Print out some statistics
		echo( "\n" );
		foreach( array( 'count' => 'Found', 'added' => 'Added',
			'skipped' => 'Skipped', 'overwritten' => 'Overwritten' ) as $var => $desc ) {
			if( $$var > 0 )
				echo( "{$desc}: {$$var}\n" );
		}
		
	} else {
		echo( "No suitable files could be found for import.\n" );
	}

} else {
	showUsage();
}

exit();

function showUsage( $reason = false ) {
	if( $reason ) {
		echo( $reason . "\n" );
	}

	echo <<<END
Imports images and other media files into the wiki
USAGE: php importImages.php [options] <dir>

<dir> : Path to the directory containing images to be imported

Options:
--extensions=<exts>	Comma-separated list of allowable extensions, defaults to \$wgFileExtensions
--overwrite		Overwrite existing images if a conflicting-named image is found
--user=<username> 	Set username of uploader, default 'Maintenance script'
--comment=<text>  	Set upload summary comment, default 'Importing image file'
--comment-file=<file>  	Set upload summary comment the the content of <file>.
--comment-ext=<ext>  	Causes the comment for each file to be loaded from a file with the same name
			but the extension <ext>.
--license=<code>  	Use an optional license template
--dry			Dry run, don't import anything
--source_wiki_url   if specified, take User and Comment data for each imported file from this URL.
                    For example, --source_wiki_url="http://en.wikipedia.org/"

END;
	exit();
}
